/*
 *	initial.c - initialization routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	InitGame()	: initialize eek systems (per game)
 *	InitLevel()	: reinitialize eek systems (per level)
 */

#include "eek.h"
#include "Random.h"



/*
 *	InitGame()	: initialize eek systems (per game)
 */
InitGame()
{
	int ret;
	extern Menu intro_menu;
 /*
 *	set up intro menus
 */
	main_menu = &intro_menu;
	StartMenus(&main_menu);
/*
 *	is tty big enough for decent game?
 */
	if (LINES < MIN_LINES || COLS < MIN_COLS) {
		touchwin(stdscr);
		erase();
		endwin();
		printf("\nScreen is too small to play eek.\n");
		ret = 0;
	} else {
/*
 *		then do all one-time screen stuff and
 *		variable initizalization
 *
 *		set playing field dimensions
 */
		NX = MAX(MIN_COLS, (COLS - 20));
		NY = MAX(MIN_LINES, LINES);
		NX = MIN(NX, MAXX);
		NY = MIN(NY, MAXY);
/*
 *		curses stuff
 */
		field = newwin(NY, NX, 0, 0);
		scorewin = newwin(2, COLS - NX, SY, NX);
		if (testing)
			testwin = newwin(14, COLS - NX, 7, NX);
		leaveok(stdscr, TRUE);
		leaveok(field, FALSE);
		leaveok(scorewin, FALSE);
		if (testing)
			leaveok(testwin, FALSE);
#if ! defined(BSD_CURSES)
		if (speed > 0)
			nodelay(field, TRUE);
		keypad(stdscr, TRUE);
		keypad(field, TRUE);
#endif /* BSD_CURSES */
/*
 *		game parameters
 */
		norm_fac = (float) (INX * INY) / (float) (NX * NY);
		nrobots /= norm_fac;
		maxrobots /= norm_fac;
		nfences /= norm_fac;
		minfences /= norm_fac;
		jug_likely = JUG_LIKELY;

		ret = 1;
	}
	return ret;
}



/*
 *	InitLevel()	: reinitialize eek systems (per level)
 */
void
InitLevel()
{
	register int x, y;

	InitRandom();
	moves = 0;
	level++;
	if ((level % 10) == 0) {
		lives++;
		UPD_LIVES();
	}
	new_level++;
	for (x = NX - 1; x > 0; x--)
		for (y = NY - 1; y > 0; y--)
			grid[x][y] = EOS;

	CreateRobots();
	CreateFences();
	CreateBerserkers();
	CreateBerserkerTraps();
	InitJuggernaut();
	CreateJuggernaut();
	CreateBlackHoles();
	InitTeleportLevel();
}
