/*
 *	jugger.c - process juggernauts
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	ProcJuggernaut()		: process juggernaut [create, move, etc]
 *	InitJuggernaut()		: decide if juggernaut creation is more likely
 *	CreateJuggernaut()	: create (if it's time) a juggernaut
 */

#include "eek.h"
#include "Random.h"


static int
	njugger = 0,			/* # of juggernauts (currently 0 or 1) */
	jug_dir;					/* direction of juggernaut */
static int nx, ny;		/* new x, new y */
static int ox, oy;		/* old x, old y */

/*
 *	ProcJuggernaut()		: process juggernaut [create, move, etc]
 */
ProcJuggernaut(ix, iy)
{
	int flag = 0;
/*
 *	if it's time, create one (but not on top of the humanoid!).
 */
	if (!njugger) {
		CreateJuggernaut();
/*
 *	or if we already have a juggernaut, move it one location.
 */
	} else {
		switch (jug_dir) {
			case 1: nx = ox + 1; ny = oy + 1; break;
			case 2: nx = ox + 1; ny = oy + 0; break;
			case 3: nx = ox + 1; ny = oy - 1; break;
			case 4: nx = ox + 0; ny = oy - 1; break;
			case 5: nx = ox - 1; ny = oy - 1; break;
			case 6: nx = ox - 1; ny = oy - 0; break;
			case 7: nx = ox - 1; ny = oy + 1; break;
			case 8: nx = ox - 0; ny = oy + 1; break;
		}
/*
 *		note that it's no longer where it was. If it hits an edge, it
 *		leaves the universe.
 */
		grid[ox][oy] = EOS;
		PUT_CH(field, oy, ox, ' ');
		if (nx <= 0 || nx >= NX - 1 || ny <= 0 || ny >= NY - 1) {
			njugger--;
		} else {
			switch (grid[nx][ny]) {
/*
 *				ol' fred just can't deal with being hit with kilotons
 *				of high-velocity mass
 */
				case HUMANOID:
					Munch(nx, ny, JUGGERNAUT);
					break;
/*
 *				for that matter, neither can anything else...
 */
				default:
					if (!InGravityWell(nx, ny)) {
						grid[nx][ny] = JUGGERNAUT;
						PUT_CH(field, ny, nx, JUGGERNAUT);
					} else {
/*
 *						except a black hole, which has megatons of mass.
 */
						if (is_worm_hole) {
							PUT_CH(field, ny, nx, ' ');
							Teleport(&ny, &nx, 0);
							PUT_CH(field, ny, nx, JUGGERNAUT);
						} else {
							njugger--;
							PUT_CH(field, ny, nx, ' ');
						}
					}
					break;
			}
		}
		ox = nx; oy = ny;
	}
	wrefresh(field);
	flag = njugger;
	return (!flag);
}


/*
 *	InitJuggernaut()	: decide if juggernaut creation is more likely
 */
void
InitJuggernaut()
{
/*
 *	what are the odds of the spontaneous emission of a juggernaut?
 *	every 3 levels, 1/jug_likely gets closer to 1 until it reaches
 *	it.
 */
	njugger = 0;
	if ((jug_likely > 1) && (level % 3))
		jug_likely--;
}



/*
 *	CreateJuggernaut()	: create (if it's time) a juggernaut
 */
void
CreateJuggernaut()
/*
 *	what are the odds of the spontaneous emission of a juggernaut?
 *	if it's time, create one (but not on top of the humanoid!),
 *	and pick a random direction.
 */
{
	if (! Random(jug_likely)) {
		njugger++;
		do {
			nx = MungedRandom(NX-2) + 1;
			ny = MungedRandom(NY-2) + 1;
		} while (grid[nx][ny] == HUMANOID);
		grid[nx][ny] = JUGGERNAUT;
		jug_dir = Random(8) + 1;
		ox = nx, oy = ny;
	}
}
